@@ -5,13 +5,21 @@ use Template;
 use FindBin '$Bin';
 use Perl::Build::Pod ':all';
 use Deploy qw/do_system older/;
+use Getopt::Long;
+my $ok = GetOptions (
+    'force' => \my $force,
+    'verbose' => \my $verbose,
+);
+if (! $ok) {
+    usage ();
+    exit;
+}
 
 # Names of the input and output files containing the documentation.
 
 my $pod = 'Create.pod';
 my $input = "$Bin/lib/JSON/$pod.tmpl";
 my $output = "$Bin/lib/JSON/$pod";
-my $force;
 
 # Template toolkit variable holder
 
@@ -31,6 +39,7 @@ my $tt = Template->new (
             0,
         ],
     },
+    STRICT => 1,
 );
 
 my @examples = <$Bin/examples/*.pl>;
@@ -38,7 +47,7 @@ for my $example (@examples) {
     my $output = $example;
     $output =~ s/\.pl$/-out.txt/;
     if (older ($output, $example) || $force) {
-	do_system ("perl $example > $output");
+	do_system ("perl -I$Bin/blib/lib -I$Bin/blib/arch $example > $output", $verbose);
     }
 }
 
@@ -47,3 +56,10 @@ $tt->process ($input, \%vars, $output, binmode => 'utf8')
 
 exit;
 
+sub usage
+{
+print <<EOF;
+--verbose
+--force
+EOF
+}
